"use strict";

// eslint-disable-next-line max-len

/* jslint vars: true, plusplus: true, devel: true, nomen: true, regexp: true, indent: 4, maxerr: 50 */
// eslint-disable-next-line no-unused-vars

/* global define, $, brackets, window */

/**
 *  File: Main.js
 *  Author: Chif <nadchif@gmail.com>
 *  Description:  This extension allows you to encode/decode selection
 */
define(function (require, exports, module) {
  'use strict';

  var encodeToASCII = require('convertors/ascii').encodeToASCII;

  var decodeFromASCII = require('convertors/ascii').decodeFromASCII;

  var encodeToURI = require('convertors/uri').encodeToURI;

  var decodeFromURI = require('convertors/uri').decodeFromURI;

  var encodeToBase64 = require('convertors/base64').encodeToBase64;

  var decodeFromBase64 = require('convertors/base64').decodeFromBase64;

  var encodeToBin = require('convertors/bin').encodeToBin;

  var decodeFromBin = require('convertors/bin').decodeFromBin;

  var encodeToHtmlEntities = require('convertors/htmlentities').encodeToHtmlEntities;

  var decodeFromHtmlEntities = require('convertors/htmlentities').decodeFromHtmlEntities;

  var encodeNlToBr = require('convertors/nl2br').encodeNlToBr;

  var decodeNlFromBr = require('convertors/nl2br').decodeNlFromBr;

  var encodeToHex = require('convertors/hexadecimal').encodeToHex;

  var decodeFromHex = require('convertors/hexadecimal').decodeFromHex;

  var encodeToMD5 = require('convertors/MD5').encodeToMD5;

  var encodeToSHA1 = require('convertors/SHA1').encodeToSHA1;

  var encodeToSHA256 = require('convertors/SHA256').encodeToSHA256;

  var encodeToSHA512 = require('convertors/SHA512').encodeToSHA512;

  var encodeToRIPEMD160 = require('convertors/RIPEMD160').encodeToRIPEMD160;

  var encodeToPHPSerial = require('convertors/PHPSerialize').encodeToPHPSerial;

  var encodeIntToPHPSerial = require('convertors/PHPSerialize').encodeIntToPHPSerial;

  var encodeFloatToPHPSerial = require('convertors/PHPSerialize').encodeFloatToPHPSerial;

  var decodeFromPHPSerial = require('convertors/PHPSerialize').decodeFromPHPSerial;

  var encodeToMorseCode = require('convertors/morsecode').encodeToMorseCode;

  var decodeFromMorseCode = require('convertors/morsecode').decodeFromMorseCode;

  var encodeUnixToUTCDate = require('convertors/unixToUTCDate').encodeUnixToUTCDate;

  var encodeToRoman = require('convertors/roman').encodeToRoman;

  var decodeFromRoman = require('convertors/roman').decodeFromRoman;
  /**
   * Register encoders and decoders here, in this format
   * {
   * title: <Title of the encoder/decoder>,
   * handler: <Function that does the conversion and returns string>
   * }
   */


  var ENCODERS_DECODERS = [{
    title: 'NL2BR',
    encodeTitle: 'New Line to BR',
    encoder: encodeNlToBr,
    decodeTitle: 'BR to New Line',
    decoder: decodeNlFromBr
  }, {
    title: 'ASCII',
    encodeTitle: 'String to ASCII',
    encoder: encodeToASCII,
    decodeTitle: 'ASCII to String',
    decoder: decodeFromASCII
  }, {
    title: 'URI',
    encoder: encodeToURI,
    decoder: decodeFromURI
  }, {
    title: 'Base 64',
    encoder: encodeToBase64,
    decoder: decodeFromBase64
  }, {
    title: 'Bin',
    encoder: encodeToBin,
    decoder: decodeFromBin
  }, {
    title: 'HTML Entities',
    encoder: encodeToHtmlEntities,
    decoder: decodeFromHtmlEntities
  }, {
    title: 'Hex',
    encoder: encodeToHex,
    decoder: decodeFromHex
  }, {
    title: 'MD5',
    encoder: encodeToMD5,
    category: 'crypto'
  }, {
    title: 'SHA256',
    encoder: encodeToSHA256,
    category: 'crypto'
  }, {
    title: 'SHA512',
    encoder: encodeToSHA512,
    encodeTitle: 'String to SHA 512',
    category: 'crypto'
  }, {
    title: 'SHA1',
    encoder: encodeToSHA1,
    encodeTitle: 'String to SHA 1',
    category: 'crypto'
  }, {
    title: 'RIPEMD160',
    encoder: encodeToRIPEMD160,
    encodeTitle: 'String to RIPEMD-160',
    category: 'crypto'
  }, {
    title: 'PHP Serial',
    encodeTitle: 'String to PHP Serial',
    encoder: encodeToPHPSerial,
    decodeTitle: 'PHP Serial to String/Int/Float',
    decoder: decodeFromPHPSerial,
    category: 'php'
  }, {
    title: 'PHP Serial 2',
    encodeTitle: 'Int to PHP Serial',
    encoder: encodeIntToPHPSerial,
    category: 'php'
  }, {
    title: 'PHP Serial 3',
    encodeTitle: 'Float to PHP Serial',
    encoder: encodeFloatToPHPSerial,
    category: 'php'
  }, {
    title: 'Morse Code',
    encodeTitle: 'String to Morse Code',
    encoder: encodeToMorseCode,
    decodeTitle: 'Morse Code to String',
    decoder: decodeFromMorseCode
  }, {
    title: 'unixToUTCDate',
    encodeTitle: 'Unix Timestamp to UTC Date',
    encoder: encodeUnixToUTCDate
  }, {
    title: 'Roman',
    // Nourma
    encodeTitle: 'Decimal to Roman',
    encoder: encodeToRoman,
    decodeTitle: 'Roman to Decimal',
    decoder: decodeFromRoman
  }];
  var CommandManager = brackets.getModule('command/CommandManager');
  var EditorManager = brackets.getModule('editor/EditorManager');
  var Menus = brackets.getModule('command/Menus');

  var convertInput = function convertInput(convertor) {
    var editor = EditorManager.getFocusedEditor();

    if (editor) {
      var selectedText = editor._codeMirror.getSelections();

      var converted = convertor(selectedText[0]);

      editor._codeMirror.replaceSelections([converted]);
    }
  };

  var ecdcMenu = Menus.getContextMenu(Menus.ContextMenuIds.EDITOR_MENU).addSubMenu('Encode/Decode Selection', 'NADCHIF_ECDC_MENU');

  var stageMenu = function stageMenu(id, categoryTitle, categoryTools) {
    var stagedMenuEncoders = [];
    var stagedMenuDecoders = [];
    var subMenu = ecdcMenu.addSubMenu(categoryTitle, "NADCHIF_ECDC_MENU_".concat(id), Menus.FIRST);
    categoryTools.forEach(function (item, index) {
      if (item.encoder) {
        CommandManager.register(item.encodeTitle ? item.encodeTitle : "String to ".concat(item.title), "ecdc_enc_".concat(id).concat(index), function () {
          convertInput(item.encoder);
        });
        stagedMenuEncoders.push({
          title: item.encodeTitle ? item.encodeTitle : "String to ".concat(item.title),
          cmd: "ecdc_enc_".concat(id).concat(index)
        });
      }

      if (item.decoder) {
        CommandManager.register(item.decodeTitle ? item.decodeTitle : "".concat(item.title, " to String"), "ecdc_dec_".concat(id).concat(index), function () {
          convertInput(item.decoder);
        });
        stagedMenuDecoders.push({
          title: item.decodeTitle ? item.decodeTitle : "".concat(item.title, " to String"),
          cmd: "ecdc_dec_".concat(id).concat(index)
        });
      }
    });
    stagedMenuEncoders.sort(function (a, b) {
      return a.title.toLowerCase() < b.title.toLowerCase() ? -1 : 1;
    }).forEach(function (item) {
      subMenu.addMenuItem(item.cmd, null);
    });

    if (stagedMenuDecoders.length > 0) {
      subMenu.addMenuDivider();
    }

    stagedMenuDecoders.sort(function (a, b) {
      return a.title.toLowerCase() < b.title.toLowerCase() ? -1 : 1;
    }).forEach(function (item) {
      subMenu.addMenuItem(item.cmd, null);
    });
  };

  var stagedMenuEncoders = [];
  var stagedMenuDecoders = [];

  var stageMenuItem = function stageMenuItem(item, index) {
    if (!item.category) {
      if (item.encoder) {
        CommandManager.register(item.encodeTitle ? item.encodeTitle : "String to ".concat(item.title), "ede".concat(index), function () {
          convertInput(item.encoder);
        });
        stagedMenuEncoders.push({
          title: item.encodeTitle ? item.encodeTitle : "String to ".concat(item.title),
          cmd: "ede".concat(index)
        });
      }

      if (item.decoder) {
        CommandManager.register(item.decodeTitle ? item.decodeTitle : "".concat(item.title, " to String"), "edd".concat(index), function () {
          convertInput(item.decoder);
        });
        stagedMenuDecoders.push({
          title: item.decodeTitle ? item.decodeTitle : "".concat(item.title, " to String"),
          cmd: "edd".concat(index)
        });
      }
    }
  };

  ENCODERS_DECODERS.forEach(stageMenuItem);
  stagedMenuEncoders.sort(function (a, b) {
    return a.title.toLowerCase() < b.title.toLowerCase() ? -1 : 1;
  }).forEach(function (item) {
    ecdcMenu.addMenuItem(item.cmd, null);
  });
  ecdcMenu.addMenuDivider();
  stagedMenuDecoders.sort(function (a, b) {
    return a.title.toLowerCase() < b.title.toLowerCase() ? -1 : 1;
  }).forEach(function (item) {
    ecdcMenu.addMenuItem(item.cmd, null);
  });
  stageMenu('php', 'PHP', ENCODERS_DECODERS.filter(function (item) {
    return item.category == 'php';
  }));
  stageMenu('crypto', 'Crypto', ENCODERS_DECODERS.filter(function (item) {
    return item.category == 'crypto';
  }));
});